#!/usr/bin/env/python
# -*- coding: utf-8 -*-
import time
import datetime


class Task(object):
    def __init__(self, schedule, fn, **options):
        self.schedule = schedule
        self.fn = fn
        self.options = options
        self.next_run = self.get_next_run(schedule)

    def get_next_run(self, schedule):
        now = datetime.datetime.now()
        delta = datetime.timedelta(minutes=int(schedule))
        return now + delta

    def run(self):
        self.fn(**self.options)

    def __repr__(self):
        return str(self.next_run)


class Scheduler(object):
    def __init__(self):
        self.tasks = []

    def add_cron(self, schedule, f, **options):
        self.tasks.append(Task(schedule, f, **options))

    def cron(self, schedule, **options):
        def decorator(f):
            self.add_cron(schedule, f, **options)
            return f
        return decorator

    def run_pending(self):
        print self.tasks
        while True:
            now = datetime.datetime.now()
            for idx, task in enumerate(self.tasks):
                if now > task.next_run:
                    t = self.tasks.pop(idx)
                    t.run()
                    self.add_cron(t.schedule, t.fn, **t.options)
            time.sleep(5)


scheduler = Scheduler()


@scheduler.cron("1")
def task1():
    print "task1"


@scheduler.cron("2")
def task2():
    print "task2"


scheduler.run_pending()



#!/usr/bin/env/python
# -*- coding: utf-8 -*-
import json
import logging
import os
import sys
import time

from slackclient import SlackClient
from . import __version__
import yaml

log = logging.getLogger(__name__)


class Gendo(object):
    def __init__(self, slack_token=None, settings=None):
        self.settings = settings or {}
        self.listeners = []
        self.scheduled_tasks = []
        self.client = SlackClient(
            slack_token or self.settings.get('gendo', {}).get('auth_token'))
        self.sleep = 0.5 or self.settings.get('gendo', {}).get('sleep')

    @classmethod
    def config_from_yaml(cls, path_to_yaml):
        with open(path_to_yaml, 'r') as ymlfile:
            settings = yaml.load(ymlfile)
            return cls(settings=settings)

    def listen_for(self, rule, **options):
        def decorator(f):
            self.add_listener(rule, f, **options)
            return f
        return decorator

    def cron(self, schedule, **options):
        def decorator(f):
            self.add_cron(schedule, f, **options)
            return f
        return decorator

    def run(self):
        running = True
        if self.client.rtm_connect():
            while running:
                time.sleep(self.sleep)
                try:
                    data = self.client.rtm_read()
                    if data and data[0].get('type') == 'message':
                        user = data[0].get('user')
                        message = data[0].get('text')
                        channel = data[0].get('channel')
                        self.respond(user, message, channel)
                except (KeyboardInterrupt, SystemExit):
                    log.info("attempting graceful shutdown...")
                    running = False
        try:
            sys.exit(0)
        except SystemExit:
            os._exit(0)

    def respond(self, user, message, channel):
        if not message:
            return
        elif message == 'gendo version':
            self.speak("Gendo v{0}".format(__version__))
            return
        for phrase, view_func, options in self.listeners:
            if phrase in message.lower():
                response = view_func(user, message, **options)
                if response:
                    if '{user.username}' in response:
                        response = response.replace('{user.username}',
                                                    self.get_user_name(user))
                    self.speak(response, channel)

    def add_listener(self, rule, view_func=None, **options):
        self.listeners.append((rule, view_func, options))

    def speak(self, message, channel):
        self.client.api_call("chat.postMessage", as_user="true:",
                             channel=channel, text=message)

    def get_user_info(self, user_id):
        user = self.client.api_call('users.info', user=user_id)
        return json.loads(user)

    def get_user_name(self, user_id):
        user = self.get_user_info(user_id)
        return user.get('user', {}).get('name')



#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from setuptools import setup, find_packages
from gendo import __version__

pip_requirements = []
for line in open('requirements.txt').readlines():
    li = line.strip()
    if not li.startswith("#"):
        pip_requirements.append(line)

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('LICENSE.txt') as readme_file:
    license = readme_file.read()

setup(
    name='gendobot',
    author='Nick Ficano',
    author_email='nficano@gmail.com',
    version=__version__,
    packages=find_packages(exclude=['tests*']),
    url='http://nickficano.com',
    description="a lightweight Slackbot framework for Python",
    zip_safe=False,
    install_requires=pip_requirements,
    long_description=readme,
    license=license,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet",
    ]
)



import mock

from gendo import Gendo

TEST_TEXT = ':muffpunch:'
TEST_TOKEN = '111'
TEST_CHANNEL = 'tests'


class TestSpeak(object):
  @mock.patch('gendo.bot.Gendo.speak')
  def test_speak(self, mock_speak):
    gendo = Gendo(__name__, TEST_TOKEN)
    gendo.speak(TEST_TEXT, TEST_CHANNEL)
    mock_speak.assert_called_with(TEST_TEXT, TEST_CHANNEL)

  @mock.patch('gendo.bot.SlackClient.api_call')
  def test_slack_call(self, mock_call):
    gendo = Gendo(__name__, TEST_TOKEN)
    gendo.speak(TEST_TEXT, TEST_CHANNEL)
    mock_call.assert_called_with(
      'chat.postMessage',
      as_user='true:',
      channel='tests',
      text=TEST_TEXT
    )
